#include "mex.h"
#include "SDPdef.h"


int MatInsertAnswer(sdpmatlabdata *, sdpdat *);
int MatReadData(sdpmatlabdata *, sdpdat *);

sdpdat  *sdat;
clock_t *stim;
FILE    *fout;
double  *dsdpmaty0;

void dsdp( sdpmatlabdata * dp ){

  int     i,j,n,beg, ierr=0,dConstraints;
  double  spar=0,spar2=0;
  sdpdat  sdt= {0};
  char    *cs,ss[40];
  clock_t tim[5]={0};
  
  stim= tim;
  sdat=&sdt;
  SetTime(stim,START);

  sdt.spc=NULL;
  SetParams(&sdt);  /* Set parameters  and adjust for matlab */
  sdt.par.saverank1sol=FALSE;
  sdt.par.printit = FALSE;
  sdt.par.save_y=FALSE;
  sdt.par.matlab=dp->inputargs;
  sdt.par.minimize=TRUE;
  sdt.par.laplacian=FALSE;
  if (dp->y0) dsdpmaty0=dp->y0;
  if ( (dp->outputargs == 4 || dp->outputargs ==2) 
       && sdt.par.findrank1sol == FALSE)  sdt.par.findrank1sol=4;
  fout = NULL;
  if (sdt.par.matlab==4 || sdt.par.matlab==6){
    sdt.par.generatey0=FALSE;
  }

  if (sdt.par.print_info) PrintHeader();

  /* Read data from the matlab structures */

  MatReadData(dp, &sdt);

  /* Create Cholesky factor structures */

  dConstraints = FindDenseMatrix(&sdt);  
  sdt.smw= wAlloc(dConstraints,sdt.n);  
  j=0;
  for (i=0; i<sdt.m; i++){
    if (sdt.useSMW[i] > 0){
      sdt.smw->dind[j]=i;
      j++;
    }
  }
  sdt.smw->ndense = dConstraints;

  CreateS(&sdt,&spar);   
  MchlSetup(&sdt);

  ierr=InitialPoint(&sdt);  /* Allocate memory and initialize it. */
  if (ierr){  
    ShutDown();  /* DeAllocate memory */
    return;
  }

  n=sdt.n;
  spar2=(sdt.sf->unnz+n)*2.0/(n*n+n+1e-10)*100.0;
  SetTime(stim,DATAIN);
  if (sdt.par.print_info){
    PrintTitle(&sdt, spar,spar2);
    PrintIt(&sdt, 0 );
  }

  DualSDP(&sdt);
  if (sdt.par.print_info) PrintEndSDP(&sdt);
  SetTime(stim,DUALALG);
  if (sdt.par.findrank1sol ) RankReduce(&sdt);  
  SetTime(stim, INTEG);

  MatInsertAnswer(dp, & sdt);

  ShutDown();  /* DeAllocate memory */
  SetTime(stim,ELAPSED);
  
  if (sdt.par.print_info) PrintEnd(&sdt,stim);

  return;

 /* End main */
}


void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
  int i,n=0,nn,m=0,mm,p=0,pp, outputargs;
  sdpmatlabdata dp;
  double *temp;

  /* Error Checking */
  if (nrhs < 3)
     mexErrMsgTxt("Too few input arguments.  See help for details. ");
  if (nrhs > 6)
     mexErrMsgTxt("Too many input arguments.  See help for details. ");

  if (nlhs < 1)
     mexErrMsgTxt("Must have at least one output argument.  See help for details. ");
  if (nlhs > 4)
     mexErrMsgTxt("Too many output arguments.  See help for details. ");



  n = mxGetN(prhs[0]);
  nn = mxGetM(prhs[0]);
  if (n != nn)
    mexErrMsgTxt("First Input argument requires square matrix.");
  if (mxIsSparse(prhs[0])==0)
    mexErrMsgTxt("First Input argument requires sparse matrix.");
  
  m = mxGetN(prhs[1]);
  nn = mxGetM(prhs[1]);
  if (n != nn)
    mexErrMsgTxt("First and Second input dimensions do not match.");
  if (mxIsSparse(prhs[1])==0)
    mexErrMsgTxt("Second Input argument requires sparse matrix.");

  mm = mxGetN(prhs[2]);
  nn = mxGetM(prhs[2]);
  if (nn*mm != m)
    mexErrMsgTxt("Second and third input dimensions do not match.");
  if (mxIsSparse(prhs[2]))
    mexErrMsgTxt("Third Input argument requires full vector.");


  if (nrhs == 4){
    mm = mxGetN(prhs[3]);
    nn = mxGetM(prhs[3]);
    if (nn*mm != m)
      mexErrMsgTxt("Fourth input dimensions do not match.");
    if (mxIsSparse(prhs[3]))
      mexErrMsgTxt("Fourth Input argument requires full vector.");
  }
  if (nrhs > 4){

    mm = mxGetN(prhs[4]);
    p = mxGetM(prhs[4]);
    if (mm != m)
      mexErrMsgTxt("Fifth and third input dimensions do not match.");
    if (mxIsSparse(prhs[4])==0)
      mexErrMsgTxt("Fifth input requires sparse matrix.");

    pp = mxGetN(prhs[3]);
    mm = mxGetM(prhs[3]);
    if (mxIsSparse(prhs[3]))
      mexErrMsgTxt("Fourth Input argument requires full vector.");
    if (pp*mm != p)
      mexErrMsgTxt("Fourth and fifth input dimensions do not match.");
    
  }

  if (nrhs == 6){
    nn = mxGetN(prhs[5]);
    mm = mxGetM(prhs[5]);
    if (nn*mm != m)
      mexErrMsgTxt("Sixth input dimensions do not match.");
    if (mxIsSparse(prhs[5]))
      mexErrMsgTxt("Sixth Input argument requires full vector.");
  }
    

  outputargs = nlhs;  dp.outputargs = outputargs;
  dp.inputargs = nrhs;
  /* Insert values and pointers into data structure */
  
  dp.n=n;
  dp.m=m;
  dp.p=p;

  temp=mxCalloc(m*(m+2) + n*n + m*(p+n), sizeof(double));
  if (temp==NULL) mexPrintf("Caution: Problem may exceed memory capacity. \n");
  else mxFree(temp);

  dp.cnnzmax=mxGetNzmax(prhs[0]);

  dp.cval=mxGetPr(prhs[0]);
  dp.cir =mxGetIr(prhs[0]);
  dp.cjc =mxGetJc(prhs[0]);

  dp.annzmax=mxGetNzmax(prhs[1]);
  dp.aval=mxGetPr(prhs[1]);
  dp.air =mxGetIr(prhs[1]);
  dp.ajc =mxGetJc(prhs[1]);

  dp.b = mxGetPr(prhs[2]);

  dp.ahatnnzmax=0;
  dp.ahatval=NULL;
  dp.ahatir =NULL;
  dp.ahatjc =NULL;

  dp.cvec = NULL;

  if (nrhs==4)
    dp.y0=mxGetPr(prhs[3]);
  else
    dp.y0=NULL;

  if (nrhs > 4){
    dp.cvec = mxGetPr(prhs[3]);

    dp.ahatnnzmax=mxGetNzmax(prhs[4]);
    dp.ahatval=mxGetPr(prhs[4]);
    dp.ahatir =mxGetIr(prhs[4]);
    dp.ahatjc =mxGetJc(prhs[4]);
  }

  if (nrhs==6)
    dp.y0=mxGetPr(prhs[5]);



  /* Create arrays for output  */
  if (plhs[0] != NULL) mxDestroyArray(plhs[0]) ;
  plhs[0] = mxCreateDoubleMatrix(m, 1, mxREAL);

  if (outputargs == 1){    
    dp.y_out=mxGetPr(plhs[0]);
    dp.dy=NULL;
    dp.xmakerc=NULL;
    dp.x=NULL;

  } else if (outputargs == 2){
    dp.y_out=mxGetPr(plhs[0]);
    dp.dy   =NULL;
    dp.xmakerc=NULL;
    if (plhs[1] != NULL) mxDestroyArray(plhs[1]) ;
    plhs[1] = mxCreateDoubleMatrix(n, 1, mxREAL) ;
    dp.x    =mxGetPr(plhs[1]);

  } else if (outputargs ==3){
    dp.y_out=mxGetPr(plhs[0]);

    if (plhs[1] != NULL) mxDestroyArray(plhs[1]) ;
    plhs[1] = mxCreateDoubleMatrix(m, 1, mxREAL) ;
    dp.dy=mxGetPr(plhs[1]);

    if (plhs[2] != NULL) mxDestroyArray(plhs[2]) ;
    plhs[2]=mxCreateDoubleMatrix(1,1,mxREAL); 

    dp.xmakerc=mxGetPr(plhs[2]);
    dp.x=NULL;

  } else if (outputargs ==4){
    dp.y_out=mxGetPr(plhs[0]);

    if (plhs[1] != NULL) mxDestroyArray(plhs[1]) ;
    plhs[1] = mxCreateDoubleMatrix(m, 1, mxREAL) ;
    dp.dy=mxGetPr(plhs[1]);

    if (plhs[2] != NULL) mxDestroyArray(plhs[2]) ;
    plhs[2]=mxCreateDoubleMatrix(1,1,mxREAL); 
    dp.xmakerc=mxGetPr(plhs[2]);

    if (plhs[3] != NULL) mxDestroyArray(plhs[3]) ;
    plhs[3] = mxCreateDoubleMatrix(n, 1, mxREAL) ;
    dp.x=mxGetPr(plhs[3]);
  }

  dsdp( &dp );

}
/* End mex Function */
  

