% DSDP Dual Positive Semidefinite Program
%   Y=DSDP(C,A,B) attempts to solve the positive semidefinite program
%   
%      minimize    trace(C*X)
%      such that   trace(a_i*a_i'*X) = B(i) i=1,...,m, X pos. semidefinite.
%
%   where A(:,i)=a_i is a vector of length n.  Matrix A is a sparse 
%   matrix with n rows and m columns.  Matrix C is a sparse matrix that
%   is intended to be symmetric.  Only the lower triangular part
%   of this matrix will be used.  Vector B is dense and has length
%   m.  This routine calls the mex function dsdp.mex* and uses a dual
%   scaling algorithm to solve the semifinite program.  The algorithm
%   uses several parameters.  Some of these parameters can be specified
%   in a specification file that must be named COPLSDP.spc.  Among
%   the options a user may specify are the stopping tolerance of the
%   algorithm and the maximum number of iterations.  See the user
%   guide for more details.  When the function DSDP is called from
%   Matlab, some specifications will automatically be set.  First,
%   the problem will always be solved as a minimization problem.
%   Second, the problem will receive all its data from matlab
%   and return only standard output.  Third, the program will not
%   create a Laplacian matrix, and fourth, the program will not read
%   a data file or write to any files, regardless of the specification 
%   file.
%
%   Other ways to call the algorithm are:
%
%   DSDP(C,A,B,Y0) specifies initial dual vector Y0.
%
%   DSDP(C,A,B,CC,AA) adds additional constraints to solve the problem
%
%      minimize    trace(C*X) + dot(CC,XX)
%      such that   trace(a_i*a_i'*X) + dot(aa_i,XX) = B(i) i=1,...,m, 
%                                       X pos. semidefinite,   X >= 0
%
%      where vector CC has dimension p and AA(:,i)=aa_i is a vector
%      of dimension p.  The matrix AA is sparse and has m columns.
%
%   DSDP(C,A,B,CC,AA,Y0) specifies initial dual vector Y0.
%
%   Y=DSDP(C,A,B) returns an approximate dual solution Y.
%
%   [Y, V] returns a vector of dimension n that is found from
%      a rank reduction procure. By default, the heuristic used
%      to find this vector is the classic maximum cut procedure
%      of Geomans and Williamson.  Other options can be specified
%      in the specification file.
%
%   [Y, DY, K] returns the information necessary to constuct the
%      approximate solution matrix X.  Let S=C- \sum (a_i*a_i')*Y(i).
%      Then X = K*inv(S)*(\sum (a_i*a_i')*DY(i) + S)*inv(S).
%
%   [Y, DY, K, V] returns a vector V found from the rank reduction 
%      procedure.
%                                                                 
%   Example maximum cut problem: 
%     > n=100; sparsity=0.3;
%     > C=sprandsym(n,sparsity); A=speye(n); b=ones(n,1); 
%     > y=dsdp(C,A,b);
%
%                                                   SB
