#include "SDPdef.h"

double VDotSD(spvec *vec1,double *dvec)
/* Compute the inner product of a sparse vector and a dense one */
{
   int i,nnz = vec1->nn0;
   double xx=0.0;
 
   if (vec1){
   for (i=0;i<nnz;i++){
     xx = xx + vec1->an[i] * dvec[vec1->ja[i]] ;
   }}

   return xx;
}

static double VDotSS(spvec *v1, spvec *v2, double *s){
/* Compute the product of a sparse vector, times a diagonal
   matrix, times a sparse vector                             */
 
  int i1,i2, ind1=0, ind2=0;
  double ans = 0.0;

  while ( (ind1<v1->nn0) && (ind2<v2->nn0) ){
    i1 = v1->ja[ind1];
    i2 = v2->ja[ind2];
    if ( i1 < i2) {
      ind1++;
    }
    else if ( i1 > i2 ) {
      ind2++;
    }
    else{
      ans = (v1->an[ind1]/s[i1]) * (v2->an[ind2]/s[i2]) + ans;
      ind1++;
      ind2++;
    }
  }
  return ans;
}


void MakeDense( spvec *sv, double * dv , int n)
/* Create an array of doubles from a vector in sparse format */
{
  int i, ind=0;;

  for (i=0; i<n; i++){
    dv[i]=0.0;
  }

  for (ind=0; ind<sv->nn0; ind++){
    dv[sv->ja[ind]] = sv->an[ind];
  }
}

void AlgorithmM(sdpdat *sdt) 
{
/* Create the Gram matrix M. It is the sum of two matrices. One part
   comes from the PSD variables while the other comes from the LP part */

  int i,j,k; 
  int  m=sdt->m;             /* number of constraints */
  int  n=sdt->n;             /* size of matrices */
  int  useSMW = sdt->smw->ndense; /* number of dense constraints */
  int  nden=useSMW;
  double t1,t2,t3;             /* temporary variables                    */
  double *s= sdt->s;           /* Dual LP vector s                       */
  double ** gram= sdt-> gram;  /* gram matrix -output                    */
  double * dgram= sdt->dgram;  /* diagonal of gram matrix -output        */
  double *rhs2= sdt->rhs2;     /* vector need for step direction -output */
  double *y=sdt->y;            /* dual vector y                          */
  double *w=sdt->wd;           /* working space of dimension n           */

  smatx *avecs=sdt->avecs;     /* Constraint data */
  smatx *amats=sdt->amats;     /* Constraint data */
  chfac *sf=sdt->sf;           /* Cholesky factor of sparse part of S    */
  smwTools * smw = sdt->smw;   /* Sherman-M-W update tools               */

  if (useSMW){
    /* Create small matrix used in SMW update procedure. */
    MakeSMWmat(sdt,y);
  }
 
  for (i=0; i<m; i++) for (j=i+1; j<m;j++) sdt->gram[i][j-i-1]=0.0; 
  for (i=0; i<m ; i++) sdt->dgram[i] = 0.0;

  for (i=0;i<m;i++){

     MakeDense(&amats->rows[i], sdt->wd2, n);
     ChlSolve(sf, sdt->wd2 ,w);

     if (nden>0){
       /* Apply Sherman-Morrison-Woodbury update formula */
       SMWupdate(smw,&amats->rows[i],NULL,w,n,1);
     }

     t1 = VDotSD(&amats->rows[i],w);
     rhs2[i] = t1;
     dgram[i] = t1*t1;

     for (j=i+1;j<m;j++){
       t1 = VDotSD(&amats->rows[j],w);
       gram[i][j-i-1]= t1*t1;
       if (gram[i][j-i-1] < 1e-30) gram[i][j-i-1]=0.0;
     }
  }

  /* Create the LP part of the matrix */
  for (i=0;i<m;i++){
    if (avecs->rows[i].nn0>0){
      t2=0.0;t3=0.0;
      for (k=0; k<avecs->rows[i].nn0; k++){
	t1 = ( avecs->rows[i].an[k] / s[avecs->rows[i].ja[k]] );
	t2 = t2+t1;
	t3 = t3+t1*t1;
      }
      rhs2[i] = rhs2[i]+t2;
      dgram[i] = dgram[i]+t3;
      
      for (j=i+1;j<m;j++){  
	/* Sparse vector dot products */
        t1 = VDotSS(&(avecs->rows[i]),&(avecs->rows[j]),s);
	gram[i][j-i-1] = gram[i][j-i-1]+t1;
      } /* End sparse vector products */
    } /* endif */
  }  /* End LP part of matrix */  

  for (i=0; i<m; i++) for (j=i+1; j<m;j++)
       if (gram[i][j-i-1] < 1e-30) gram[i][j-i-1]=0.0;
 

}  /* End routine */

void algmrhs(sdpdat *sdt) 
{
/* Create the Gram matrix M. It is the sum of two matrices. One part
   comes from the PSD variables while the other comes from the LP part */

  int  i,j,k; 
  int  m=sdt->m;             /* number of constraints */
  int  n=sdt->n;             /* size of matrices */
  int  useSMW = sdt->smw->ndense; /* number of dense constraints */
  int  nden=useSMW;
  double t1,t2,t3;             /* temporary variables                    */
  double *s= sdt->s;           /* Dual LP vector s                       */
  double *rhs2= sdt->rhs2;     /* vector need for step direction -output */
  double *w=sdt->wd;           /* working space of dimension n           */
  smatx *avecs=sdt->avecs;     /* Constraint data */
  smatx *amats=sdt->amats;     /* Constraint data */
  chfac *sf=sdt->sf;           /* Cholesky factor of sparse part of S    */
  smwTools * smw = sdt->smw;   /* Sherman-M-W update tools               */

  if (useSMW){
    /* Create small matrix used in SMW update procedure. */
    MakeSMWmat(sdt,sdt->y);
  }
 

  for (i=0;i<m;i++){

     MakeDense(&amats->rows[i], sdt->wd2, n);
     ChlSolve(sf, sdt->wd2 ,w);

     if (nden>0){
       /* Apply Sherman-Morrison-Woodbury update formula */
       SMWupdate(smw,&amats->rows[i],NULL,w,n,1);
     }

     t1 = VDotSD(&amats->rows[i],w);
     rhs2[i] = t1;

  }

  /* Create the LP part of the matrix */
  for (i=0;i<m;i++){
    if (avecs->rows[i].nn0>0){
      t2=0.0;t3=0.0;
      for (k=0; k<avecs->rows[i].nn0; k++){
	t1 = ( avecs->rows[i].an[k] / s[avecs->rows[i].ja[k]] );
	t2 = t2+t1;
	t3 = t3+t1*t1;
      }
      rhs2[i] = rhs2[i]+t2;
      
    } /* endif */
  }  /* End LP part of matrix */  

}  /* End routine */

