#include "SDPdef.h"

void CreateS1(sdpdat *sdt, int *nnz, int *adj)
{
/* Pass in an array of pointers to integers.  Allocate enough
space for the integers and store them in the array 'snnz'.
The number of integers for each row is stored in 'rnnz'    */

  symat *cmat=sdt->cmat;
  int n = sdt->n;
  
  long int i,j,k,h,q,t,m = sdt->m,totalnnz;

  q = n*(n-1)/2;

  /* Allocate a dense upper half matrix of integers
     and initialize the integers  to 0 */
  for (i=0; i<q; i++){
    adj[i] = 0;
  }

  /* Check matrix C for nonzero entries. */
  t=0;
  for (i=0;i<n-1;i++){
    for (j=0;j<cmat->roff[i].nn0;j++){
      adj[t+cmat->roff[i].ja[j]-i-1] = 1;
    }
    t = t+(n-i-1);

  }
  
  /* Check constraint vectors. */
  for (i=0;i<m;i++){
    if (sdt->useSMW[i]==0){
      for (j=0; j<sdt->amats->rows[i].nn0; j++){
         h=sdt->amats->rows[i].ja[j];
         t=q-(n-h)*(n-h-1)/2;

         for (k=j+1;k<sdt->amats->rows[i].nn0; k++){
           adj[t+sdt->amats->rows[i].ja[k]-h-1] = 1; 
         }   
	 
      }
    }
  }

  totalnnz = 0;
  for (i=0; i<q; i++){
    if (adj[i] == 1){
      totalnnz++;
    }
  }
  *nnz=totalnnz;
}


void CreateS2( int *rnnz, int *snnz, int *adj, int n)
{

  long int i,j,k,t,count;
  
  k=0;
  t=0;
  for (i=0; i<n; i++){
    count = 0;
    for (j=0;j<n-i-1;j++){
      if (adj[t+j] == 1){
	  snnz[k]=j+i+1;
	  count++;
	  k++;
      }
    }
    rnnz[i] = count;
    t = t+(n-i-1);
  }

} 

void CreateS(sdpdat *sdt,double *spars){

  int *rnnz , nnz, *snnz, *adj;

  rnnz = iAlloc(sdt->n+1,"Setup");
  adj = iAlloc(sdt->n*((sdt->n)-1)/2+1, "Setup");
  
  CreateS1(sdt, &nnz, adj); 
  snnz = iAlloc(nnz, "Setup");
 
  CreateS2(rnnz, snnz, adj,sdt->n);
  /* SB function that creates S structure */

  iFree(&adj);

  SymbProc(rnnz,snnz,sdt);

  iFree(&snnz);
  iFree(&rnnz);

  *spars=(2.0*(nnz)+sdt->n)/(sdt->n*(sdt->n))*100.0;

}

