#include "SDPdef.h"
/*
void dposv( char * , int *m , int * , double *, int *,  int *);
*/
int DualSDP(sdpdat *sdt)
{

  int i,notPSD,rhofound,useSMW=sdt->smw->ndense;
  int gotpobj=FALSE, attempt;
  double rhoscale[20],np=sdt->n+sdt->p;
  double newP=sdt->potential,newgap;
  double tempgap=1.0,gapnow,rhotemp;

  /* Main loop for the dual positive semidefinite program       */
  /* Quit when an approximate solution is found or the maximum
     iteration count is exceeded                                */

  sdt->itnow = 0;
  sdt->pfail=0;

  if (sdt->par.generatey0) sdt->rho=1.2*np;
  else {
    sdt->oldrho=10.0; sdt->pfail=0;
    sdt->rho=GetDualRho(sdt);
  }


  while ( (sdt->itnow < sdt->par.maxiter)  &&
       	  (sdt->rgap > sdt->par.rgaptol ) &&
	  (sdt->dobj < sdt->par.dobj_stop) )
  {

    sdt->itnow++;

    /* Compute the Gram matrix M and rhs */

    AlgorithmM(sdt);
    
    /* Compute step directions  'dy1' and 'dy2' */
    ChlFact(sdt->mf,sdt->iw,sdt->rw,TRUE);
    
    DoubleCopy(sdt->b, sdt->yt2, sdt->m);
    ChlSolve(sdt->mf, sdt->yt2, sdt->dy1);
    DoubleCopy(sdt->rhs2, sdt->yt2, sdt->m);
    ChlSolve(sdt->mf, sdt->yt2, sdt->dy2);

    /* If necessary, initialize primal objective */
    if ( gotpobj == FALSE) Initial_X(sdt,&gotpobj);


    /* Determine paramenter rho */

    SetRhoScale(sdt, rhoscale, sdt->rho);
    rhofound = FALSE; sdt->prho = 0.0;
    gapnow=sdt->gap; tempgap=gapnow*10;
    /* Look for a feasible, primal objective value */
    for (i=0;i<sdt->par.primal_checks;i++){  /* Inexact search for best rho */
      rhotemp = rhoscale[i]; 
      notPSD = TRUE;
      AddVecs(rhotemp/gapnow,sdt->dy1,-1.0,sdt->dy2,sdt->dy, sdt->m);
      newgap = (Dot(sdt->dy,sdt->rhs2,sdt->m)+np)*(gapnow/rhotemp);
      AddVecs(1.0,sdt->y,-1.0,sdt->dy,sdt->ytemp, sdt->m);
      /* Check if vector x is pos.   */
      if (rhofound==FALSE || newgap<sdt->gap || gotpobj==FALSE){
	if ( LPVariables(sdt, sdt->ytemp, sdt->s) > 0 ){
	  /* Check if X is pos def.      */
	  if (useSMW){
	    NumChol(sdt, sdt->ytemp, FALSE);
	    notPSD = CheckPSD(sdt,sdt->ytemp,sdt->sf);
	  }
	  else if (NumChol(sdt, sdt->ytemp, TRUE)==TRUE ) {notPSD = FALSE;}
	}
      }
      if (notPSD ==FALSE && newgap<sdt->gap){
	/* if new primal point found   */
	DoubleCopy(sdt->y, sdt->xmakery, sdt->m);
	DoubleCopy(sdt->dy, sdt->xmakerdy, sdt->m);
	sdt->xmakerc = gapnow/rhotemp;
	sdt->gap = newgap;
	sdt->pobj = sdt->dobj + newgap;
	rhofound = TRUE;
	gotpobj = TRUE;	sdt->gotpobj=TRUE;
	sdt->prho=rhotemp;
	sdt->pfail = 0;
	tempgap=newgap;
      }
      
      if ( gotpobj == FALSE){
	DoubleCopy(sdt->y, sdt->xmakery, sdt->m);
	DoubleCopy(sdt->dy, sdt->xmakerdy, sdt->m);
	sdt->xmakerc = gapnow/rhotemp;
      }
     
      else if ((notPSD ==FALSE) && (rhofound==FALSE && newgap < tempgap)){
	rhofound=TRUE;
	tempgap=newgap;
	sdt->prho=rhotemp;
	sdt->gap = newgap;
	sdt->pobj = sdt->dobj + newgap;
      }
      
    }
    if (rhofound == FALSE) sdt->pfail ++;
    
    sdt->rho=GetDualRho(sdt);

    sdt->oldgap = sdt->gap;
	  
    AddVecs(sdt->rho/sdt->gap,sdt->dy1,-1.0,sdt->dy2,sdt->dy, sdt->m);
    AddVecs(sdt->rho/sdt->gap,sdt->b,-1.0,sdt->rhs2,sdt->yt2, sdt->m);
    sdt->pnorm = sqrt( fabs(Dot(sdt->yt2,sdt->dy,sdt->m)));

    notPSD = TRUE; attempt=0;
    while (notPSD){
      /* Select a stepsize           */
      GetStepSize(sdt, sdt->itnow,attempt,1); attempt++;
      /* Compute a new vector y      */
      AddVecs(1.0,sdt->y,sdt->alpha[0]/sdt->pnorm,sdt->dy,sdt->ytemp, sdt->m);
      newgap=sdt->pobj - Dot(sdt->ytemp, sdt->b, sdt->m);
      /* Compute s and check if pos. */
      if (LPVariables(sdt,sdt->ytemp,sdt->s) > 0){
	/* Create and factor the sparse part of matrix S */
	if (useSMW == 0){
	  if (NumChol(sdt, sdt->ytemp, TRUE) == TRUE) notPSD = FALSE;
	}else
	  NumChol(sdt, sdt->ytemp, FALSE);
	  notPSD = CheckPSD(sdt,sdt->ytemp,sdt->sf);
      } /* end if */
    } /* end while loop */
    
    DoubleCopy(sdt->ytemp,sdt->y, sdt->m);     /* Accept new y  */

    /* Update dual objective value and duality gap */
    sdt->oldrho=sdt->rho; sdt->pnormold=sdt->pnorm;
    sdt->dobj = Dot(sdt->y, sdt->b, sdt->m);
    sdt->gaphist[1]=sdt->gaphist[0];


    sdt->gap = sdt->pobj - sdt->dobj;
    if (gotpobj == FALSE && sdt->gap<0){
      sdt->gap = fabs(sdt->dobj)+1.0;
      sdt->pobj = sdt->dobj+sdt->gap; 
    }
    sdt->gaphist[0] = sdt->gap;
    sdt->rgap = sdt->gap / (fabs(sdt->dobj+sdt->pobj)/2.0 + 1.0);
    sdt->potential = newP;
    sdt->ynorm=0; 
    for (i=0;i<sdt->m;i++) sdt->ynorm=max(fabs(sdt->y[i]),sdt->ynorm);

    /* Print information */
    if (sdt->par.print_info){
      if (gotpobj == FALSE){
	printf("*");
      } else printf(" "); 
    }
    if (fout){
      if (gotpobj == FALSE){
	fprintf(fout,"*");
      } else fprintf(fout," "); 
    }
 
    PrintIt(sdt, sdt->itnow);
  }

  return (sdt->itnow);
}



void DoubleCopy(double *v1, double *v2, int nn)
{
  int i;
  for (i=0; i<nn; i++){
    v2[i] = v1[i];
  }
}

double Dot(double *v1, double *v2, int nn){
  int i;
  double ans=0.0;

  for (i=0; i<nn; i++){
    ans = v1[i]*v2[i] + ans;
  }
  return ans;
}

void AddVecs(double c1,double *v1, 
	     double c2, double *v2, 
	     double *ans, int nn){
  int i;
  for (i=0; i<nn; i++){
    ans[i] = c1*v1[i] + c2*v2[i];
  }
}





double EvalPotential(sdpdat *sdt,double dualitygap){
  /* This function uses the factorization of the dual matrix
     to evaluate the dual potential function.	*/
  int i,n = sdt->n, p=sdt->p;
  double rho, potential;
	
  rho = (n+p)*1.3;

  potential = rho * log(dualitygap);

  for (i=0; i<n; i++){
    potential = potential - log(sdt->sf->diag[i]);
  }
  for (i=0; i<p; i++){
    potential = potential - log(sdt->s[i]);
  }
  return potential;
}



