#include "SDPdef.h"
#include <stdio.h>

static int SortThem(int *ind1, int *ind2, double *dval, int n){

  int i1tmp, i2tmp, i,j,k,last=n,ierr=0;
  double dvtmp;

  for(j=1;j<last;j++){
    for (i=j; i>0; i--){

      if ( ind1[i]>ind1[i-1] || (ind1[i]==ind1[i-1] && ind2[i]>=ind2[i-1]) )
	break;
      else{	
	i1tmp = ind1[i];
	i2tmp = ind2[i];
	dvtmp  = dval[i];
	
	ind1[i]=ind1[i-1];
	ind2[i]=ind2[i-1];
	dval[i]=dval[i-1];
	
	ind1[i-1]=i1tmp;
	ind2[i-1]=i2tmp;
	dval[i-1]=dvtmp;
      }

    }      
    if (ind1[i]==ind1[i-1] && ind2[i]==ind2[i-1]){
      printf("Warning multiple constraint detected: %d %d %12.10f\n",
	     ind1[i]+1,ind2[i]+1,dval[i]);
      ind1[i]=-10000; ierr=1;
      for (k=i; k<n-1; k++){
	i1tmp = ind1[k];
	i2tmp = ind2[k];
	dvtmp  = dval[k];
	
	ind1[k]=ind1[k+1];
	ind2[k]=ind2[k+1];
	dval[k]=dval[k+1];
	
	ind1[k+1]=i1tmp;
	ind2[k+1]=i2tmp;
	dval[k+1]=dvtmp;
      }
    last--;
    }
  }

  return ierr;
}


int DataIn(sdpdat* sdt){

  char ss[256], thisline[256], shortstr[100];
  FILE *fp;
  int i,k,ierr;
  int cnnz=0,annz=0,ahatnnz=0,cvecnnz=0,coffnnz=0;
  int n=0,m=0,p=0,pp=0;
  int row,col,ind,constr;
  int card=0,line=0,*crows,nn0,roffnn0,ppcount;
  double rtmp;

  /*  thisline = calloc(256,1);
  ss    = calloc(256,1);
  */

  if(!thisline) { printf("\n Out of memory"); exit(0); }
  if(!ss   ) { printf("\n Out of memory"); exit(0); }

  fp=fopen(sdt->ss,"r");
  if (!fp){
     printf("Cannot read file %s !",sdt->ss);
     exit(0);
  }


  /* Check the file for to see if all the sections are there
     and to do some error checking 
  */
  card = 0; line=0;
  while(!feof(fp) && card<6){
    fgets(thisline,200,fp); line++;

    if( (card==0) && (strstr(thisline,"ROWS")) ){card=1; continue;}
    else if( (card==1) && (strstr(thisline,"POBJM")) ){card=2; continue;}
    else if((card==2)&&(strstr(thisline,"POBJV"))){card=3; continue;}
    else if((card==3)&&(strstr(thisline,"CONM"))){card=4; continue;}
    else if((card==4)&&(strstr(thisline,"CONV"))){card=5; continue;}
    else if((card==5) &&
	    ((strstr(thisline,"ENDDATA") || (strstr(thisline,"ENDATA"))))){
	      card=6; continue;
    }
    else if((strstr(thisline,"ROWS")||strstr(thisline,"POBJM") ||
	     strstr(thisline,"POBJV")||strstr(thisline,"CONM") ||
	     strstr(thisline,"CONV")||strstr(thisline,"ENDATA")||
	     strstr(thisline,"ENDDATA")) && (card<6)){ 
      printf("Error line %d.  Sections out of order. \n",line); 
      exit(0);
    } 
    else if ( card==0){
      printf("Data Error, line %d: Empty line. \n",line);
      exit(0);
    } 
    else if (card==1) {
      if (sscanf(thisline," %50s %d %lf %s",shortstr,&constr,&rtmp,ss)==3){
	if (constr < 1){
	  printf("Data Input Error, line %d: Nonpositive index.\n",line);
	  exit(0);
	}
	m++;
	continue;
      } else {
	printf("Data Input Error, line %d: Expecting 3 entries.\n",line); 
	exit(0);
      }
    } 
    else if (card==2) {
      if (sscanf(thisline," %d %d %lf %s",&row,&col,&rtmp,ss)==3){
	if (row<1 || col<1){
	  printf("Data Error, line %d, nonpositive index.\n",line);
	  exit(0);
	}
	n=max(max(row,col),n);
	if (row<col){coffnnz++;} 
	cnnz++;
	continue;
      } else {
	printf("Data Input Error, line %d: Expecting 3 entries \n",line); 
	exit(0);
      }
    } 
    else if (card==3){
      if (sscanf(thisline,"%d %lf %s\n",&ind,&rtmp,ss)==2){
	if ( ind < 1 ){
	  printf("Data Error: line %d, nonpositive index.\n",line);
	  exit(0);
	}
	cvecnnz++;
	p=max(p,ind);
      } else {
	printf("Data Input Error, line %d: Expecting 2 entries  \n",line); 
	exit(0);
      }
      continue;
    } 
    else if (card==4){
      if (sscanf(thisline,"%d %d %lf %s\n",&constr,&ind,&rtmp,ss)==3){
	if (constr<1 || ind<1){
	  printf("Error: line %d, nonpositive index.\n",line);
	  exit(0);
	}
	n=max(n,ind);
	annz++; 
	continue;
      } else {
	printf("Data Input Error, line %d: Expecting 3 entries.\n",line); 
	exit(0);
      }
    } else if (card==5){
      if (sscanf(thisline,"%d %d %lf %s\n",&constr,&ind,&rtmp,ss)==3){
	if (constr<1 || ind<1){
	  printf("Error: line %d, nonpositive index.\n",line);
	  exit(0);
	}
	p=max(p,ind);
	ahatnnz++; 
	continue;
      } 
      else {
	printf("Data Input Error, line %d: Expecting 3 entries.\n",line); 
	exit(0);
      }
    } 
  }

  if (card!=6) 
    printf("Input data error: Input data section missing or out of order. \n");
  if (card==0){ 
    printf("Input data error: Cannot find 'ROWS' section \n"); 
    exit(0);
  } else if (card==1){ 
    printf("Input data error: Cannot find 'POBJM' section \n"); 
    exit(0);
  } else if (card==2){ 
    printf("Input data error: Cannot find 'POBJV' section \n"); 
    exit(0);
  } else if (card==3){ 
    printf("Input data error: Cannot find 'CONM' section \n"); 
    exit(0);
  } else  if (card==4){ 
    printf("Input data error: Cannot find 'CONV' section \n"); 
    exit(0);
  } else  if (card==5){ 
    printf("Input data error: Cannot find 'ENDDATA' \n"); 
    exit(0);
  }

  fseek(fp,0,SEEK_SET);
  line = 0;

  /* Assign memory */  
  sdt->rtyp=(char*)malloc(m*sizeof(char));
  sdt->b  = dAlloc(m,NULL);
  if (!sdt->rtyp || !sdt->b){
    printf(" out of space.\n");
    exit(0);
  }
  for (i=0; i<m; i++) sdt->rtyp[i] = 'U';
  sdt->m=m;
  sdt->n=n;
  sdt->p=p;

  fgets(thisline,200,fp); line++;

  /* Read dual objective vector B and type of constraints */
  for (i=0; i<m; i++){
    fgets(thisline,200,fp); line++;
    sscanf(thisline," %50s %d %lf ",shortstr,&constr,&rtmp);
     
    if (constr > m) {
      printf("Error line %d: , In ROWS section there don't appear ",line);
      printf("to be %d constraints. Check numbering\n",constr);
      exit(0);
    }
    if (constr < 1) {
      printf("Error line %d: ",line);
      printf("Use positive constraint numbers.\n");
      exit(0);
      }
    if (sdt->rtyp[constr-1] != 'U'){
      printf("Error line %d: duplicate constraint %d.\n",line,constr);
      exit(0);
    }
    if ( !strcmp(shortstr,"E") ) sdt->rtyp[constr-1]='E';
    else if ( !strcmp(shortstr,"L") ) sdt->rtyp[constr-1]='L';
    else if ( !strcmp(shortstr,"G") ) sdt->rtyp[constr-1]='G';
    
    else{
      printf("Error: line %d. Expecting 'E' or 'L' or 'G'. \n",line);
      exit(0);
    }
    
    sdt->b[constr-1] = rtmp; 
    
  }

  /* Read the primal objective matrix */
  sdt->cmat = SymAlloc(sdt->n,coffnnz,NULL);
  crows=malloc(coffnnz*sizeof(int));
  fgets(thisline,200,fp); line++;
  
  nn0=0;
  for (i=0; i<cnnz; i++){
    fgets(thisline,200,fp); line++;
    sscanf(thisline,"%d %d %lf\n",&row,&col,&rtmp);
    if (row==col){sdt->cmat->diag[row-1]=rtmp;}
    else if (row<col){
      crows[nn0]=row-1;
      sdt->cmat->roff->ja[nn0]=col-1;
      sdt->cmat->roff->an[nn0]=rtmp;
      nn0++;
    }
    else{
      printf("Warning, line %d, lower triangular element ignored. \n",line);
    }
  }

  /* Order the stuff */
  ierr=SortThem(crows, sdt->cmat->roff->ja,sdt->cmat->roff->an,nn0);
  if (ierr) printf("Repeated entries in POBJM \n");
  /* Assign sparse vector pointers */
  k=0;
  for (i=0; i<n; i++){
    roffnn0=0;
    sdt->cmat->roff[i].ja=&(sdt->cmat->roff->ja[k]);
    sdt->cmat->roff[i].an=&(sdt->cmat->roff->an[k]);
    while(k<nn0 && crows[k]==i){
	k++; roffnn0++;
    }
    sdt->cmat->roff[i].nn0=roffnn0;
  }

  free(crows);

  /* Read the primal objective vector */
  pp=0;
  for (i=0; i<m; i++) if (sdt->rtyp[i] == 'L' || sdt->rtyp[i] == 'G') pp++;;
  sdt->cvec=calloc((p+pp),sizeof(double));
  fgets(thisline,200,fp); line++;
  for (i=0; i<cvecnnz; i++){
    fgets(thisline,200,fp); line++;
    if (sscanf(thisline,"%d %lf %s\n",&ind,&rtmp,ss)==2){
      sdt->cvec[ind-1]=rtmp;
    }
  }

  /* Read in the constraint matrices */
  sdt->amats=mAlloc(m,annz,NULL);
  crows=malloc(annz*sizeof(int));
  fgets(thisline,200,fp); line++;

  nn0=0;
  for (i=0; i<annz; i++){
    fgets(thisline,200,fp); line++;
    if (sscanf(thisline,"%d %d %lf\n",&row,&col,&rtmp)==3){
      if (row>m || row < 1){
	printf(" Error line %d:  Constraint %d does ",line,row);
	printf("not appear in ROWS\n");
	exit(0);
      }
      crows[nn0]=row-1;
      sdt->amats->rows->ja[nn0]=col-1;
      sdt->amats->rows->an[nn0]=rtmp;
      nn0++;
    }
    else{
      printf("Error, line %d: Expecting 3 arguments \n",line);
      exit(0);
    }
  }
  /* Order the stuff */
  ierr=SortThem(crows, sdt->amats->rows->ja,sdt->amats->rows->an,annz);
  if (ierr) printf("Repeated entries in CONM \n");
  /* Assign spare vector pointers */
  k=0;
  for (i=0; i<m; i++){
    roffnn0=0;
    sdt->amats->rows[i].ja=&(sdt->amats->rows->ja[k]);
    sdt->amats->rows[i].an=&(sdt->amats->rows->an[k]);
    while(k<annz && crows[k]==i){
      k++; roffnn0++;
    }
    sdt->amats->rows[i].nn0=roffnn0;
  }

  free(crows);
 
  /* Read in the constraint vectors */
  sdt->avecs=mAlloc(m,ahatnnz+pp,NULL);
  crows=malloc((ahatnnz+pp)*sizeof(int));
  fgets(thisline,200,fp); line++;
  nn0=0;

  for (i=0; i<ahatnnz; i++){
    fgets(thisline,200,fp); line++;
    if (sscanf(thisline,"%d %d %lf %s \n",&row,&col,&rtmp,ss)==3){
      if (row>m || row < 1){
	printf(" Error line %d:  Constraint %d does ",line,row);
	printf("not appear in ROWS\n");
	exit(0);
      }
      crows[nn0]=row-1;
      sdt->avecs->rows->ja[nn0]=col-1;
      sdt->avecs->rows->an[nn0]=rtmp;
      nn0++;
    }
    else{
      printf("Warning, line %d: Expecting 3 arguments \n",line);
    }
  }

  ppcount=0;
  for (i=0; i<m; i++){

    if (sdt->rtyp[i] == 'L'){
      crows[nn0+ppcount]=i;
      sdt->avecs->rows->ja[ahatnnz+ppcount] = p+ppcount;
      sdt->avecs->rows->an[ahatnnz+ppcount] = 1.0;
      ppcount++;
    }
    if (sdt->rtyp[i] == 'G'){
      crows[nn0+ppcount]=i;
      sdt->avecs->rows->ja[ahatnnz+ppcount] = p+ppcount;
      sdt->avecs->rows->an[ahatnnz+ppcount] = -1.0;
      ppcount++;
    }
  }

  /* Order the stuff */
  ierr=SortThem(crows, sdt->avecs->rows->ja,sdt->avecs->rows->an,ahatnnz);
  if (ierr) printf("Repeated entries in CONV \n");
  /* Assign spare vector pointers */
  k=0;
  for (i=0; i<m; i++){
    roffnn0=0;
    sdt->avecs->rows[i].ja=&(sdt->avecs->rows->ja[k]);
    sdt->avecs->rows[i].an=&(sdt->avecs->rows->an[k]);

    while(k<ahatnnz+pp && crows[k]==i){
      k++; roffnn0++;
    }
    sdt->avecs->rows[i].nn0=roffnn0;
  }

  sdt->pnnz = coffnnz+n;
  sdt->amnz = annz;
  sdt->avnz = ahatnnz+pp;
  sdt->p=sdt->p+pp;

  free(crows);

  sdt->useSMW= iAlloc(m, "Setup");
 
  return TRUE;
 
}



int CheckArgs(int argc,char *argv[],char *ss)
{
  int i,id;
  
  id = 0;
  for (i=1; i<argc; i++) 
  {
    if(strstr(argv[i],ss)) 
        {
      argv[i]+=2;
      if (argv[i][0] == ':') 
          {
        argv[i]++;
        id=i;
        break;
      }
      if (argv[i][0]) 
          {
        id=i;
        break;
      }
      id = i+1;
      if (argv[id][0] == '-')
        id = 0;
      break;
    }
  }
  return id;
} /* Checkargs */











