
#include "SDPdef.h"
void UserRule(double *, double *, int );

typedef struct {

  int useSMW;
  double *wvec1;
  double *wvec2;
  double *wvec3;
  double *wvec4;
  double *val;
  double *ei;
  int * rank;
  orderVec *vorder;
  smwTools *smw2;
} roundWork;
 
int GetRow(sdpdat *, roundWork*,int, double *);
int GetVecHat(sdpdat *, roundWork *,double *);
int GetBigEigenVector(sdpdat *, roundWork *,double *);
int ColorIt(sdpdat *, roundWork *, double *);


static double VDot(double *v1, double *v2, int n){
  int i;
  double ans=0.0;

  for (i=0; i<n; i++){  ans = ans+v1[i]*v2[i]; }
  return ans;
}

static double SDVDot(spvec *vec1,double *dvec)
/* Compute the inner product of a sparse vector and a dense one */
{
   int i,nnz = vec1->nn0;
   double xx=0.0;
 
   if (vec1){
   for (i=0;i<nnz;i++){
     xx = xx + vec1->an[i] * dvec[vec1->ja[i]] ;
   }}

   return xx;
}

double EvalObjective( symat  *objM,  
                      double *nvec,  
                      int    n)
/* Evaluate the quadratic function xQx where Q  is a symmetric
   sparse matrix and x is a vector.  This is done by calculating
   the inner product of Q with the outer product of the vector  */
{  double answer=0; 
   int i,j,k,nnzr; 
  
   for (i=0;i<n;i++)
   {
      answer = answer + objM->diag[i] * nvec[i] * nvec[i];
      nnzr = objM->roff[i].nn0;
      for (j=0;j<nnzr;j++)
      {
         k = objM->roff[i].ja[j];
         answer = answer + objM->roff[i].an[j] * nvec[i] * nvec[k] * 2;
      }
   } 
   return (answer);
} 
 
static int cut_comp(const void *e1,const void *e2)
/* Used in qsort routine to sort the structure 'orderVec' by its value  */
{ 
   double d1, d2;
   orderVec *a1,*a2;
   a1=(orderVec*)e1;
   a2=(orderVec*)e2;
   d1 = *(a1->val); 
   d2 = *(a2->val); 
 
   if (d1<d2) 
      return (-1); 
   else if (d1>d2) 
      return (1); 
   return(0); 
}

static orderVec *OrdAlloc(int  len, char *info)
{
  orderVec *r=NULL;
  
  if (len) {
    r=(orderVec*)calloc(len,sizeof(orderVec));
    if (!r) ExitProc(OutOfSpc,info);
  }
  return r;
} /* OrdAlloc */

static void OrdFree(orderVec **x)
{
  orderVec *r=*x;
  
  if (r) {
    free(r);
    *x=NULL;
  }
} /* ordFree */


static int MaxCutRule(double *vhat, double *vsol, int n){
  int i;

  for (i=0;i<n;i++){
    if (vhat[i]>0) vsol[i]=1.0;
    else vsol[i]=-1.0;
  }
  return 0;
}

static int BoxqpRule(double *vhat, double *vsol, int n){
  int i;

  for (i=0;i<n;i++){
    vsol[i] = sign(vhat[i])*fabs(vsol[i]); 
  }
  return 0;
}

static int SizeCutRule(double *vhat, double *vsol, int n, 
		       orderVec *vhatorder, int cut){
  int i;

  for (i=0;i<n;i++){
    vhatorder[i].index=i;
    vhatorder[i].val = &(vhat[i]);
  }
  
  qsort( (void *)vhatorder, n, sizeof(orderVec), cut_comp);
    
  if ( *(vhatorder[n/2].val) > 0){
    for ( i=0;i<(n-cut)/2;i++)
      vsol[vhatorder[i].index] = -1.0;
    for ( i=(n-cut)/2; i<n; i++)
      vsol[vhatorder[i].index] = 1.0;
  }
  
  else{
    for ( i=0;i<(n+cut)/2;i++)
      vsol[vhatorder[i].index] = 1.0;
    for ( i=(n+cut)/2; i<n; i++) 
      vsol[vhatorder[i].index] = -1.0;
  }

  /*  Could also implement for nonbinary variables 
  if ( *(vhatorder[n/2].val) > 0){
    for ( i=0;i<(n-cut)/2;i++)
      vsol[vhatorder[i].index] = -fabs(vsol[vhatorder[i].index]);
    for ( i=(n-cut)/2; i<n; i++)
      vsol[vhatorder[i].index] = fabs(vsol[vhatorder[i].index]);
  }
  
  else{
    for ( i=0;i<(n+cut)/2;i++)
      vsol[vhatorder[i].index] = fabs(vsol[vhatorder[i].index]);
    for ( i=(n+cut)/2; i<n; i++) 
      vsol[vhatorder[i].index] = -fabs(vsol[vhatorder[i].index]);
  }
  */

  return 0;
}

static int MaxCoverRule(double *vhat, double *vsol, int n, sdpdat *sdt)
    /* This routine creates feasible solutions for the stable set problems.
	It assumes the constraints |v_i + v_j + v_n| are written after the
	boolean constraints and checks for violated contraints.  
	If it finds a violated constraint, it will change the sign of either
	v_i or v_j, whichever is farther from v_n.                      */
{
  int i,changes=0;
  spvec *spv;
  
  for (i=0;i<sdt->n; i++){
    if (vhat[i]<0) vsol[i] = -1;
    else vsol[i] = 1;
  }
  for (i=0;i<sdt->m; i++){ if (sdt->amats->rows[i].nn0 == 3){
    spv = &(sdt->amats->rows[i]);
    if ( (sign(vsol[spv->ja[0]]) == sign(vsol[spv->ja[2]])) && 
	 (sign(vsol[spv->ja[1]]) == sign(vsol[spv->ja[2]])) ){
      changes++;
      if ( (vhat[spv->ja[0]]-vhat[spv->ja[2]]) > 
	   (vhat[spv->ja[1]]-vhat[spv->ja[2]]) ){
	vsol[spv->ja[0]] = -vsol[spv->ja[0]];
      }
      else{
	vsol[spv->ja[1]] = -vsol[spv->ja[1]];
      }
    }
  }}

  /*
  for (i=0;i<sdt->m; i++){ if (sdt->amats->rows[i].nn0 >= 3){
    spv = &(sdt->amats->rows[i]);
    if (sdt->amats->rows[i].nn0 == 3){
    } else {

      while (SDVDot(spv,vsol) > sqrt(fabs(sdt->b[i])) ){
	inner_count++;
	for (j=0; j<sdt->amats->rows[i].nn0 - 1; j++){}
      }
    }      
  */

  return changes;
}
  



void GetRandVec(double *randVec, int n) 
/* This routine obtains a vector from the n dimensional
   unit sphere.  Actually, if the vector were to be normalized
   to length one, it would be a randomly chosen vector from the
   unit sphere.                                                 */
   
{  int i; 
   double scal,x;
   scal=pow(2.0,15)-1.0;
   scal = RAND_MAX;
   for (i=0;i<n;i++){ 
     x = (( rand())/scal - .5);
     randVec[i] = tan(3.1415926*x); 
   }
}



void RankReduce(sdpdat *sdt)
/* This routine generates a specific cut or solution to the original, 
   unrelaxed problem. */ 
{
  int i,j=0,m=sdt->m, p=sdt->p, n=sdt->n;
  int  intcut,cutsize=0,cut=0;
  int method=sdt->par.findrank1sol,useSMW, ndense=sdt->smw->ndense;
  double one=1.0,thisObj;
  double *vhat,*vsol,*xvar;
  double temp;
  chfac *tempcf;
  probtype ptype=sdt->par.prtype;
  roundWork rw;
  symat *cmat=sdt->cmat;


  if (sdt->gotpobj == FALSE){
    method=2;
    if (sdt->par.print_info){
      printf("No feasible matrix could be found.\n"); 
      printf("Using indefinite matrix. \n");
    }
    if (fout){
      printf("No feasible matrix could be found.\n"); 
      printf("Using indefinite matrix. \n");
    }
  }

  useSMW = ndense;
  
  vsol    = sdt->vsol;
  vhat    = sdt->vhat;

  rw.useSMW=ndense;
  rw.wvec1=sdt->wd;
  rw.wvec2=sdt->wd2;
  rw.wvec3=dAlloc(n,"Insufficient memory to store cut");
  rw.wvec4=dAlloc(n,"Insufficient memory to store cut");
  rw.val=dAlloc(n,"Insufficient memory to store cut");
  rw.rank=iAlloc(n,"Insufficient memory to store cut");
  rw.vorder=OrdAlloc(sdt->n,"Insufficient memory to store cut");
  rw.ei=dAlloc(n,"Insufficient memory to store cut");
  rw.smw2=wAlloc(sdt->smw->ndense,sdt->n);
   
  srand(clock());   /*  Used to generate random vectors */
  
  /* Free the large gram matrix, do symbollic work */
  CfcFree(&sdt->mf); sdt->mf=NULL;
  free(sdt->gram);
  sdt->gram=NULL;
  sdt->dgram = NULL;

  if (sdt->par.prtype == ecut){
    intcut = 1;
    for (i=0; i<m; i++){
      if (sdt->amats->rows[i].nn0 > intcut){
	intcut = sdt->amats->rows[i].nn0;
	cut = (int) sqrt(fabs(sdt->b[i])+1e-8);
      }
    }
    cut = min(max(0,cut),n);
    cutsize=cut;
    if ((n-cut)%2 == 1) cut ++;
    printf("Trying to satisfy e^T v = %d \n\n",cut);
  }

  if (sdt->par.prtype == maxcover){
    intcut = 0;
    for (i=0; i<m; i++){
      if ( sdt->amats->rows[i].nn0 == 1)
	continue;
      else if ( sdt->amats->rows[i].nn0 == 3){
	if (sdt->amats->rows[i].ja[2] != sdt->n-1){
	  if (sdt->par.print_info){
	    printf("Warning: third nonzero index in constraint %d", i+1);
	    printf(" is inconsistent with this heuristic. \n");
	  }
	  if (fout){
	    printf("Warning: third nonzero index in constraint %d", i+1);
	    printf(" is inconsistent with this heuristic. \n");
	  }
	  intcut = 5;
	}}
      else intcut = sdt->amats->rows[i].nn0;
    }
    if (intcut)
      printf("Warning: problem data doesn't match expectations. \n");
    intcut=0;
  }
  /* Initialize the vector solution to the the correct magnitude. 
     Look for constraints that bound the diagonal of the primal matrix
     Calculate linear part and subtract it from the rhs */
   
   if (sdt->p > 0){
     xvar = dAlloc(sdt->p,"Insufficient memory to store cut"); 
     AddVecs(1.0,sdt->xmakery,-1.0,sdt->xmakerdy,sdt->ytemp, sdt->m);
     LPVariables(sdt, sdt->ytemp, xvar);
     LPVariables(sdt, sdt->xmakery, sdt->s);
     for (i=0; i<p; i++){
       xvar[i]=xvar[i]/(sdt->s[i]*sdt->s[i])*sdt->xmakerc;
     }
     
   }

   for (i=0; i<n; i++) rw.val[i] = 1.0;

   if (ptype == boxqp || ptype == user){
     for (i=0;i<sdt->m;i++){
       if (sdt->amats->rows[i].nn0 == 1  
	   /* && sdt->aa.amat[i].ja[0] == i  Part2 optional */){
	 if (sdt->p == 0) rw.val[sdt->amats->rows[i].ja[0]] = sqrt(fabs((sdt->b[i])));
	 else{
	   rw.val[sdt->amats->rows[i].ja[0]]=
	     sqrt(fabs( sdt->b[i] - VDotSD(&(sdt->avecs->rows[i]),xvar) ));
	 }
       }  
     }
   }   

   
   
   if ( (method==3 || method == 4) || ptype!=mincolor ){
     /* Use random vectors  to calculate a solution */

     tempcf=sdt->sf;
     sdt->sf = sdt->rf;
     CreateS(sdt, &temp);

     AddVecs(1.0,sdt->xmakery,-1.0,sdt->xmakerdy,sdt->ytemp, sdt->m);
     NumChol(sdt,sdt->ytemp,FALSE);
     RVectorCoeffs(sdt, sdt->ytemp);

     rw.smw2->ndiag = sdt->smw->ndiag;
     for (i=0; i<ndense+rw.smw2->ndiag; i++){
       rw.smw2->v2[i] = sdt->smw->v2[i];
     }
     for (i=ndense+rw.smw2->ndiag; i<2*ndense; i++){
       rw.smw2->v2[i]=0.0;
     }

     for (i=0; i<ndense; i++){
       for (j=0; j<sdt->n; j++){
      	 rw.smw2->upd[i][j]=sdt->smw->upd[i][j];
       }
     }
     for (i=ndense; i<2*ndense; i++){
       for (j=0; j<sdt->n; j++) rw.smw2->upd[i][j]=0.0;
       if ( i<ndense + rw.smw2->ndiag ){
	 rw.smw2->upd[i][sdt->smw->ndInd[i-ndense]]=1.0;
       }
     }       

     sdt->rf = sdt->sf;
     sdt->sf= tempcf;
   }

   NumChol(sdt,sdt->xmakery,FALSE);
   if (CheckPSD(sdt,sdt->xmakery,sdt->sf)) printf("Nearly Singular \n");
   if (useSMW) MakeSMWmat(sdt,sdt->xmakery);


   if ( (method==3 || method == 4) && ptype != mincolor){
     /* Use randomized method to calculate a solution */

     for (i=0; i<n; i++){
       /* Calculate vhat */
       for (j=0;j<n;j++) vsol[j] = rw.val[j];
       GetVecHat(sdt, &rw,vhat);

       /* Use vhat to find a solution */
       if (ptype == maxcut ) MaxCutRule(vhat, vsol, n);
       else if (ptype == boxqp ) BoxqpRule(vhat, vsol, n);
       else if (ptype==maxcover) intcut+=MaxCoverRule(vhat, vsol, n, sdt);
       else if (ptype==ecut) SizeCutRule(vhat, vsol, n, rw.vorder, cutsize);
       else if (ptype==user) UserRule(vhat,vsol,n);
       else MaxCutRule(vhat, vsol, n);


       thisObj = EvalObjective(sdt->cmat,vsol,n);
       if (i==0 || thisObj < sdt->bestRObj){
	 sdt->bestRObj=thisObj; 
	 if (thisObj < sdt->bestObj){
	   sdt->bestObj = thisObj;
	   DoubleCopy(vsol, sdt->bestv, n);
	 }
       }
     } 
   }



   if ( (method == 2 || method == 3) && ptype!=mincolor ){
     /* Use the rows of the primal solution X to find vector solution */
     for (i=0; i<n; i++){       /* Calculate row i of X */

       for (j=0;j<n;j++) vsol[j] = rw.val[j];
       GetRow(sdt, &rw,i, vhat); printf(" %4.4e\n",vhat[i]);

       /* Use the row to find a solution */
       if (ptype == maxcut ) MaxCutRule(vhat, vsol, n);
       else if (ptype == boxqp ) BoxqpRule(vhat, vsol, n);
       else if (ptype==maxcover) MaxCoverRule(vhat, vsol, n, sdt);
       else if (ptype==ecut) SizeCutRule(vhat, vsol, n, rw.vorder, cutsize);
       else if (ptype==user) UserRule(vhat,vsol,n);
       else MaxCutRule(vhat, vsol, n);
	 

       thisObj = EvalObjective(sdt->cmat,vsol,n); 
       if (i==0 || thisObj < sdt->bestHObj){ 
	 sdt->bestHObj=thisObj;
	 if (thisObj < sdt->bestObj){
	   sdt->bestObj=thisObj;
	   DoubleCopy(vsol, sdt->bestv, n);
	 }
       }
     }
   }

   if ((method == 1 || method==3) && ptype != mincolor){

     GetBigEigenVector(sdt, &rw,vhat);

     if (ptype == maxcut ) MaxCutRule(vhat, vsol, n);
     else if (ptype == boxqp ) BoxqpRule(vhat, vsol, n);
     else if (ptype==maxcover) MaxCoverRule(vhat, vsol, n, sdt);
     else if (ptype==ecut) SizeCutRule(vhat, vsol, n, rw.vorder, cutsize);
     else if (ptype==user) UserRule(vhat,vsol,n);
     else MaxCutRule(vhat, vsol, n);

     sdt->eigObj = EvalObjective(sdt->cmat,vsol,n); 

     if (sdt->eigObj < sdt->bestObj){
       sdt->bestObj=sdt->eigObj;
       DoubleCopy(vsol, sdt->bestv, n);
     }
   }


   if (ptype == mincolor){

     ColorIt(sdt, &rw, vsol);
  }

   if (ptype ==maxcover){
     if (sdt->par.print_info) 
       printf("Average number of vertices changed: %4.2f \n",intcut*1.0/n);
     if (fout) 
       fprintf(fout,"Average number of vertices changed: %4.2f \n",intcut*1.0/n);
   }

   if (ptype != mincolor){
     PrintIntObj(n,sdt->eigObj, sdt->bestHObj, sdt->bestRObj, &(sdt->par) ); 
   }
   if (sdt->par.saverank1sol)
     PrintVecSol(sdt->bestv,n,sdt->ss, sdt->par.prtype);


   if (sdt->p > 0) dFree(&xvar); 

   dFree(&rw.val); 
   dFree(&rw.wvec4); 
   OrdFree(&rw.vorder);
   iFree(&rw.rank);
   dFree(&rw.wvec3);
   dFree(&rw.ei);
   wFree(&rw.smw2);

} 


void ATdyTv(sdpdat *sdt, double *co, double *v1, double* ans){
/* This function computes the vector ans=A^T(co)*v1.  Recall
   that the matrices A_i have a rank one structure.         */

  int i,j,m=sdt->m,n=sdt->n,nnz;
  double temp;
  spvec *svec;

  for (i=0; i<n; i++){
    ans[i]= 0.0;
  }

  for (i=0; i<m; i++){
    svec = &(sdt->amats->rows[i]);
    temp = VDotSD(svec, v1);
    nnz = svec->nn0;
    for (j=0; j<nnz; j++){
      ans[svec->ja[j]] += temp*co[i]*(svec->an[j]);
    }
  }
}


int GetRow(sdpdat *sdt, roundWork *rw,int row, double *rowi){

  int j, n=sdt->n, trow=row;
  double one=1.0;
  spvec spei;

  spei.nn0 = 1;   spei.an=&one;  spei.ja=&trow;

  memset(rw->ei,0,sizeof(double)*n);
  rw->ei[row]=1.0 ;
  
  ChlSolve(sdt->sf, rw->ei, rw->wvec1);
  if (rw->useSMW) SMWupdate(sdt->smw,&spei,NULL,rw->wvec1,n,TRUE);
  
  ATdyTv(sdt,sdt->xmakerdy,rw->wvec1, rw->wvec2);
  
  DoubleCopy(rw->wvec2,rw->ei,n);
  
  ChlSolve(sdt->sf, rw->ei, rw->wvec3);
  if (rw->useSMW) SMWupdate(sdt->smw,NULL,rw->wvec2,rw->wvec3, n,FALSE);
  AddVecs(1.0, rw->wvec1, 1.0, rw->wvec3, rowi, n);

  for (j=0; j<n; j++) rowi[j]=rowi[j]*sdt->xmakerc;

  return 0;
}


int GetVecHat(sdpdat *sdt, roundWork *rw,double *vhat){

  int j,n=sdt->n;
  int  ndense=sdt->smw->ndense;
  double temp;
  probtype ptype=sdt->par.prtype;

  GetRandVec(rw->wvec4,n);
  /* For user function, normalize the vector */
  if (ptype==user){
    temp=sqrt(sdt->xmakerc/VDot(rw->wvec4,rw->wvec4,n));
    for (j=0; j<n; j++) rw->wvec4[j]=rw->wvec4[j]*temp;
  }
  
  
  for (j=0; j<ndense+rw->smw2->ndiag; j++){
    temp=rw->smw2->v2[j]*VDot(rw->wvec4,rw->smw2->upd[j],sdt->n);
    AddVecs(1.0, rw->wvec4, temp, rw->smw2->upd[j], rw->wvec4, n);
  }
  
  DoubleCopy(rw->wvec4,rw->wvec1,n);
  GetUhat(sdt->rf,rw->wvec1,rw->wvec2);
  DoubleCopy(rw->wvec2,rw->wvec1,n);
  
  ChlSolve(sdt->sf,rw->wvec2, rw->wvec3);
  if (rw->useSMW) SMWupdate(sdt->smw,NULL,rw->wvec1,rw->wvec3,n,FALSE);
  DoubleCopy(rw->wvec3, vhat, n);
  
}

int GetBigEigenVector(sdpdat *sdt, roundWork *rw,double *vhat){
  
  int i,j,n=sdt->n;
  double temp;
  double oldeig=0,bigeig=0;
  
  GetRandVec(vhat,n);
  temp=sqrt(fabs(VDot(vhat,vhat,n)));
  
  for (j=0; j<n/2+1; j++){
    if (oldeig > 0 && bigeig/oldeig > 0.9999) break;
    
    for (i=0; i<n; i++) vhat[i]=vhat[i]/(temp);
    
    DoubleCopy(vhat,rw->ei,n);
    
    ChlSolve(sdt->sf, vhat, rw->wvec1);
    if (rw->useSMW) SMWupdate(sdt->smw,NULL,rw->ei,rw->wvec1, n,FALSE);
    
    ATdyTv(sdt,sdt->xmakerdy,rw->wvec1, rw->wvec2);
    
    DoubleCopy(rw->wvec2,rw->ei,n);      
    
    ChlSolve(sdt->sf, rw->ei, rw->wvec3);
    if (rw->useSMW) SMWupdate(sdt->smw,NULL,rw->wvec2,rw->wvec3,n,FALSE);
    AddVecs(1.0, rw->wvec1, 1.0, rw->wvec3, vhat, n);
    
    oldeig = bigeig;
    temp=fabs(VDot(vhat,vhat,n));
    bigeig= sqrt(temp*sdt->xmakerc); temp=sqrt(temp);
  }
  
}


int ColorIt(sdpdat *sdt, roundWork *rw, double *coloring){
  
  int *rnnz,*adj,*snnz;
  int i,j,k,h,hh,q,totalnnz,totnnz,n=sdt->n,m=sdt->m;
  int count,color=0,uncolored=n;
  int maxdegree=-1, maxvertex=-1;
  
  rnnz = iAlloc(sdt->n+1,"Setup");
  adj = iAlloc(sdt->n*sdt->n+1, "Setup");
  
  q = n*n;
  
  /* Allocate a dense upper half matrix of integers
     and initialize the integers  to 0 */
  for (i=0; i<q; i++){
    adj[i] = 0;
  }
  
  
  /* Check constraint vectors. */
  for (i=0;i<m;i++){
    for (j=0; j<sdt->amats->rows[i].nn0; j++){
      for (k=0; k<sdt->amats->rows[i].nn0; k++){
	h=sdt->amats->rows[i].ja[j];
	hh=sdt->amats->rows[i].ja[k];
	adj[n*h+hh] = 1;
	adj[n*hh+h] = 1;
      }
    }
  }   
  totalnnz = 0;
  for (i=0; i<q; i++){
    if (adj[i] == 1){
      totalnnz++;
    }
  }
  snnz = iAlloc(totalnnz, "Setup");
  
  totnnz=0; rnnz[0]=0;
  for (i=0; i<n; i++){
    count = 0;
    for (j=0;j<n;j++){
      if (adj[n*i+j] == 1){
	snnz[totnnz]=j;
	count++; totnnz++;
      }
    }
    rnnz[i+1] = count;
  }
  
  
  iFree(&adj);

  for (i=1;i<n+1; i++) rnnz[i]+=rnnz[i-1];
  for (i=0;i<n;i++) coloring[i]=0.0;

  while (uncolored > 0){
    color++;
    for (i=0;i<n;i++) rw->rank[i]=0;
    
    for (i=0;i<n;i++){
      if (coloring[i]<0.5){
	for (j=rnnz[i]; j<rnnz[i+1];j++){
	  if (coloring[snnz[j]] < 0.5) rw->rank[i]++;
	}
      }
    } /* END FOR */
    
    maxvertex=-1;maxdegree=-1;
    for (i=0;i<n;i++){
      if (coloring[i] <0.5 && rw->rank[i]>maxdegree){ 
	maxvertex=i; maxdegree=rw->rank[i];
      }
    }
    
    /* Get row */
    GetRow(sdt, rw,maxvertex, rw->wvec4);       
    /* Got row */
    
    coloring[maxvertex]=color; uncolored--;
    
    for (i=0;i<n;i++){
      if (coloring[i]>0.5) rw->wvec4[i]=-2.0;
    }
    for (j=rnnz[maxvertex];j<rnnz[maxvertex+1];j++) rw->wvec4[snnz[j]]=-2.0;
    
    for (i=0;i<n;i++){
      rw->vorder[i].index=i;
      rw->vorder[i].val = &(rw->wvec4[i]);
    }
    qsort( (void *)rw->vorder, n, sizeof(orderVec), cut_comp);
    
    for (i=n-1; i>=0;i--){
      if (rw->wvec4[rw->vorder[i].index] > -1.0){
	coloring[rw->vorder[i].index]=color; uncolored--;
	for (j=rnnz[rw->vorder[i].index];j<rnnz[rw->vorder[i].index+1];j++) 
	  rw->wvec4[snnz[j]]=-2.0;
      }
    }
  }
  
  iFree(&snnz);
  iFree(&rnnz);
  
  DoubleCopy(coloring, sdt->bestv, n);

  if (sdt->par.print_info) printf("Number of colors: %d \n",color);
  if (fout) fprintf(fout,"Number of colors: %d \n",color);
  
  return 0;
}













